@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /*
- *  liboaf: A library for accessing oafd in a nice way.
+ *  bonobo-activation: A library for accessing bonobo-activation-server.
  *
  *  Copyright (C) 1999, 2000 Red Hat, Inc.
  *  Copyright (C) 2000, 2001 Eazel, Inc.
@@ -31,7 +31,8 @@
 #include <bonobo-activation/bonobo-activation-i18n.h>
 #include <bonobo-activation/bonobo-activation-init.h>
 #include <bonobo-activation/bonobo-activation-private.h>
-
+#include <bonobo-activation/Bonobo_ActivationContext.h>
+#include <bonobo-activation/bonobo-activation-client.h>
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE 1
@@ -48,6 +49,11 @@
 #include <stdlib.h>
 #include <fcntl.h>
 
+/* If you have a strange unix, you get odd hard coded limits */
+#ifndef PATH_MAX
+#  define PATH_MAX 1024
+#endif
+
 static GSList *registries = NULL;
 
 typedef struct
@@ -268,9 +274,34 @@ existing_check (const BonoboActivationBaseService *base_service, struct SysServe
 	return CORBA_OBJECT_NIL;
 }
 
+void
+bonobo_activation_base_service_debug_shutdown (CORBA_Environment *ev)
+{
+        int     i;
+        GSList *l, *instances;
+        struct SysServerInstance *ssi;
+
+	for (i = 0; activatable_servers[i].name; i++) {
+
+                instances = activatable_servers[i].instances;
+                activatable_servers[i].instances = NULL;
+
+                for (l = instances; l; l = l->next) {
+                        ssi = l->data;
+
+                        CORBA_Object_release (ssi->already_running, ev);
+                        g_free (ssi->username);
+                        g_free (ssi->hostname);
+                        g_free (ssi->domain);
+                        g_free (ssi);
+                }
+                g_slist_free (instances);
+        }
+}
+
 static void
 bonobo_activation_existing_set (const BonoboActivationBaseService *base_service, struct SysServer *ss,
-	          CORBA_Object obj, CORBA_Environment *ev)
+                                CORBA_Object obj, CORBA_Environment *ev)
 {
 	GSList *link;
 	struct SysServerInstance *ssi;
@@ -302,6 +333,11 @@ bonobo_activation_existing_set (const BonoboActivationBaseService *base_service,
 		CORBA_Object_release (ssi->already_running, ev);
 		ssi->already_running = obj;
 	}
+
+        /* FIXME: all this code is unneccesarily abstract & buggy with it */
+        if (!strcmp (base_service->name, "IDL:Bonobo/ActivationContext:1.0")) {
+                bonobo_activation_register_client (obj, ev);
+        }
 }
 
 static GSList *activator_list = NULL;
@@ -332,7 +368,7 @@ bonobo_activation_base_service_activator_add (BonoboActivationBaseServiceActivat
 
 static CORBA_Object
 bonobo_activation_activators_use (const BonoboActivationBaseService *base_service, const char **cmd,
-		    int fd_arg, CORBA_Environment *ev)
+                                  int fd_arg, CORBA_Environment *ev)
 {
 	CORBA_Object retval = CORBA_OBJECT_NIL;
 	GSList *link;
@@ -348,10 +384,11 @@ bonobo_activation_activators_use (const BonoboActivationBaseService *base_servic
 	return retval;
 }
 
-static CORBA_Object
-bonobo_activation_service_get_internal (const BonoboActivationBaseService *base_service,
-                                        gboolean              existing_only,
-                                        CORBA_Environment    *ev)
+CORBA_Object
+bonobo_activation_internal_service_get_extended (
+        const BonoboActivationBaseService *base_service,
+        gboolean              existing_only,
+        CORBA_Environment    *ev)
 {
 	CORBA_Object retval = CORBA_OBJECT_NIL;
 	int i;
@@ -397,6 +434,7 @@ bonobo_activation_service_get_internal (const BonoboActivationBaseService *base_
 			retval = race_condition;
 		} else if (!CORBA_Object_is_nil (retval, &myev)) {
 			bonobo_activation_base_service_set (base_service, retval, &myev);
+                        CORBA_Object_release (race_condition, &myev);
                 }
 	}
 
@@ -428,22 +466,14 @@ bonobo_activation_service_get (const BonoboActivationBaseService *base_service)
         
         CORBA_exception_init (&ev);
         
-        obj = bonobo_activation_service_get_internal (base_service, FALSE, &ev);
+        obj = bonobo_activation_internal_service_get_extended (
+                base_service, FALSE, &ev);
 
         CORBA_exception_free (&ev);
 
         return obj;
 }
 
-CORBA_Object
-bonobo_activation_internal_service_get_extended (const BonoboActivationBaseService *base_service,
-                                   gboolean              existing_only,
-                                   CORBA_Environment    *ev)
-{
-        return bonobo_activation_service_get_internal (base_service, existing_only, ev);
-}
-
-
 /*****Implementation of the IOR registration system via plain files ******/
 static int lock_fd = -1;
 
@@ -453,10 +483,12 @@ rloc_file_lock (const BonoboActivationBaseServiceRegistry *registry,
 {
 	char *fn;
 	struct flock lock;
+        int retval;
+        char *err;
 
-        fn = g_strdup_printf ("/tmp/orbit-%s/oaf-register.lock", g_get_user_name ());
+        fn = g_strdup_printf ("/tmp/orbit-%s/bonobo-activation-register.lock", g_get_user_name ());
 
-	while ((lock_fd = open (fn, O_CREAT | O_RDONLY, 0700)) < 0) {
+	while ((lock_fd = open (fn, O_CREAT | O_RDWR, 0700)) < 0) {
 		if (errno == EEXIST) {
 #ifdef HAVE_USLEEP
 			usleep (10000);
@@ -483,8 +515,14 @@ rloc_file_lock (const BonoboActivationBaseServiceRegistry *registry,
 		lock.l_len = 1;
 		lock.l_pid = getpid ();
 
-		while (fcntl (lock_fd, F_SETLKW, &lock) < 0
+		while ((retval = fcntl (lock_fd, F_SETLKW, &lock)) < 0
 		       && errno == EINTR) /**/;
+
+                if (retval < 0) {
+                        /* FIXME: need to report this error in a better way. */
+                        err = strerror (errno);
+                        g_warning ("Failed to acquire lock: %s\n.", err);
+                }
 	}
 
         g_free (fn);
@@ -495,15 +533,6 @@ rloc_file_unlock (const BonoboActivationBaseServiceRegistry *registry,
                   gpointer user_data)
 {
         struct flock lock;
-#if 0
-	char *fn;
-
-
-	fn = bonobo_activation_alloca (sizeof ("/tmp/orbit-%s/oaf-register.lock") + 32);
-	sprintf (fn, "/tmp/orbit-%s/oaf-register.lock", g_get_user_name ());
-
-	unlink (fn);
-#endif
 
 	if (lock_fd >= 0) {
 
@@ -636,7 +665,8 @@ rloc_file_unregister (const BonoboActivationBaseServiceRegistry *registry,
 	fn = g_strdup_printf ("/tmp/orbit-%s/reg.%s-%s",
                               uname,
                               namecopy,
-                              base_service->session_name ? base_service->session_name : "local");
+                              base_service->session_name ?
+                              base_service->session_name : "local");
 	unlink (fn);
 
 	fn2 = g_strdup_printf ("/tmp/orbit-%s/reg.%s", uname, namecopy);
@@ -667,8 +697,3 @@ bonobo_activation_rloc_file_register (void)
 {
 	bonobo_activation_base_service_registry_add (&rloc_file, 0, NULL);
 }
-
-
-
-
-
